@@ -1,4 +1,4 @@
-from typing import Union
+from typing import Union, Optional
 
 import numpy as np
 from scipy.constants import mu_0
@@ -17,33 +17,57 @@ def biot_savart_2d(
     positions: np.ndarray,
     current_densities: np.ndarray,
     z0: float = 0,
+    areas: Optional[np.ndarray] = None,
     length_units: str = "um",
     current_units: str = "uA",
+    vector: bool = True,
 ) -> np.ndarray:
-    """Returns the z-component of the magnetic field (in tesla) from a sheet of current
-    located at vertical positon ``z0`` (in units of ``lengt_units``). The current is
+    """Returns the magnetic field (in tesla) from a sheet of current located at
+    vertical positon ``z0`` (in units of ``length_units``). The current is
     parameterized by a set of ``current_densities`` (in units of
     ``current_units / length_units``) and x-y ``positions`` (in units of
     ``length_units``), and the field is evaluated at coordinates ``(x, y, z)``.
 
+    .. math::
+
+        \\mu_0H_x(\\vec{r}) &= \\frac{\\mu_0}{4\\pi}\\int_S
+        \\frac{J_y(\\vec{r}')(\\vec{r}-\\vec{r}')\\cdot\\hat{z}}
+        {|\\vec{r}-\\vec{r}'|^3}\\,\\mathrm{d}^2r'\\\\
+        \\mu_0H_y(\\vec{r}) &= \\frac{\\mu_0}{4\\pi}\\int_S
+        -\\frac{J_x(\\vec{r}')(\\vec{r}-\\vec{r}')\\cdot\\hat{z}}
+        {|\\vec{r}-\\vec{r}'|^3}\\,\\mathrm{d}^2r'\\\\
+        \\mu_0H_z(\\vec{r}) &= \\frac{\\mu_0}{4\\pi}\\int_S
+        \\frac{J_x(\\vec{r}')(\\vec{r}-\\vec{r}')\\cdot\\hat{y}
+        - J_y(\\vec{r}')(\\vec{r}-\\vec{r}')\\cdot\\hat{x}}
+        {|\\vec{r}-\\vec{r}'|^3}\\,\\mathrm{d}^2r'
+
+
+    where :math:`\\vec{r}=(x, y, z)` and :math:`\\vec{r}'=(x', y', z_0)`.
+
     Args:
-        x: x-coordinate(s) (in meters) at which to evaluate the field.
+        x: x-coordinate(s) at which to evaluate the field.
             Either a scalar or vector with shape ``(n, )``.
-        y: y-coordinate(s) (in meters) at which to evaluate the field.
+        y: y-coordinate(s) at which to evaluate the field.
             Either a scalar or vector with shape ``(n, )``.
-        z: z-coordinate(s) (in meters) at which to evaluate the field. Either a scalar
+        z: z-coordinate(s) at which to evaluate the field. Either a scalar
             or vector with shape ``(n, )``.
-        positions: Coordinates ``(x0, y0)`` (in meters) of the current sheet,
+        positions: Coordinates ``(x0, y0)`` of the current sheet,
             shape ``(m, 2)``.
-        current_densities: 2D current density ``(Jx, Jy)`` in units of amps / meter,
-            shape``(m, 2)``.
+        current_densities: 2D current density ``(Jx, Jy)``, shape``(m, 2)``.
         z0: Vertical (z) position of the current sheet.
+        areas: Vertex areas for ``positions`` in units of ``length_units**2``. If None,
+            the ``positions`` are triangulated to calculate vertex areas.
         length_units: The units for all coordinates.
         current_units: The units for current values. The ``current_densities`` are
             assumed to be in units of ``current_units / length_units``.
+        vector: Return the full vector magnetic field (shape ``(n, 3)``) rather
+            than just the z-component (shape ``(n, )``).
 
     Returns:
-        Magnetic field ``Bz`` in tesla evaluated at ``(x, y, z)``, shape ``(n, )``
+        Magnetic field in tesla evaluated at ``(x, y, z)``. If ``vector`` is True,
+        returns the vector magnetic field :math:`\\mu_0\\vec{H}` (shape ``(n, 3)``).
+        Otherwise, returns the the :math:`z`-component, :math:`\\mu_0H_z`
+        (shape ``(n,)``).
     """
     # Convert everything to base units: meters and amps / meter.
     to_meter = ureg(length_units).to("m").magnitude
@@ -65,13 +89,24 @@ def biot_savart_2d(
     dx = np.subtract.outer(x, x0)
     dy = np.subtract.outer(y, y0)
     dz = np.subtract.outer(z, z0 * np.ones_like(x0))
-    # Triangulate the current sheet to assign an effective area to each vertex.
-    triangles = Delaunay(positions).simplices
-    areas = mass_matrix(positions, triangles)
+    if areas is None:
+        # Triangulate the current sheet to assign an effective area to each vertex.
+        triangles = Delaunay(positions).simplices
+        areas = mass_matrix(positions, triangles)
+    else:
+        areas = areas * to_meter**2
     # Evaluate the Biot-Savart integral.
-    return (mu_0 / (4 * np.pi)) * (
-        areas * (Jx * dy - Jy * dx) / (dx**2 + dy**2 + dz**2) ** (3 / 2)
-    ).sum(axis=1)
+    pref = (mu_0 / (4 * np.pi)) * areas * (dx**2 + dy**2 + dz**2) ** (-3 / 2)
+    Jx_dy = np.einsum("ij, ij, j -> i", pref, dy, Jx)
+    Jy_dx = np.einsum("ij, ij, j -> i", pref, dx, Jy)
+    Bz = Jx_dy - Jy_dx
+    if not vector:
+        return Bz
+    Jy_dz = np.einsum("ij, ij, j -> i", pref, dz, Jy)
+    Jx_dz = np.einsum("ij, ij, j -> i", pref, dz, Jx)
+    Bx = Jy_dz
+    By = -Jx_dz
+    return np.stack([Bx, By, Bz], axis=1)
 
 
 def SheetCurrentField(
@@ -91,7 +126,7 @@ def SheetCurrentField(
 
     .. math::
 
-        \\mu_0H_z(\\vec{r})=\\frac{\\mu_0}{2\\pi}\\int_S
+        \\mu_0H_z(\\vec{r})=\\frac{\\mu_0}{4\\pi}\\int_S
         \\frac{J_x(\\vec{r}')(\\vec{r}-\\vec{r}')\\cdot\\hat{y}
         - J_y(\\vec{r}')(\\vec{r}-\\vec{r}')\\cdot\\hat{x}}
         {|\\vec{r}-\\vec{r}'|^3}\\,\\mathrm{d}^2r',
@@ -119,4 +154,5 @@ def SheetCurrentField(
         z0=z0,
         length_units=length_units,
         current_units=current_units,
+        vector=False,
     )
