@@ -1,5 +1,6 @@
 from collections import defaultdict
 import warnings
+import inspect
 import itertools
 from contextlib import contextmanager
 from typing import Optional, Union, Tuple, List, Dict, Sequence
@@ -248,6 +249,7 @@ def plot_streams_layer(
     cmap: str = "magma",
     levels: int = 101,
     colorbar: bool = True,
+    filled: bool = True,
     **kwargs,
 ) -> Tuple[plt.Axes, Optional[Colorbar]]:
     """Plots the stream function for a single layer in a Device.
@@ -265,6 +267,7 @@ def plot_streams_layer(
         cmap: Name of the matplotlib colormap to use.
         levels: Number of contour levels to used.
         colorbar: Whether to add a colorbar to the plot.
+        filled: If True, plots filled contours.
 
     Returns:
         matplotlib axis and Colorbar if one was created (None otherwise)
@@ -282,8 +285,8 @@ def plot_streams_layer(
     if isinstance(units, str):
         units = device.ureg(units).units
     stream = (solution.streams[layer] * device.ureg(solution.current_units)).to(units)
-
-    im = ax.tricontourf(x, y, triangles, stream.magnitude, cmap=cmap, levels=levels)
+    plot_func = ax.tricontourf if filled else ax.tricontour
+    im = plot_func(x, y, triangles, stream.magnitude, cmap=cmap, levels=levels)
     ax.set_xlabel(f"$x$ [${length_units:~L}$]")
     ax.set_ylabel(f"$y$ [${length_units:~L}$]")
     ax.set_title(f"$g$ ({layer})")
@@ -305,6 +308,7 @@ def plot_streams(
     cmap: str = "magma",
     levels: int = 101,
     colorbar: bool = True,
+    filled: bool = True,
     **kwargs,
 ) -> Tuple[plt.Figure, np.ndarray]:
     """Plots the stream function for multiple layers in a Device.
@@ -321,6 +325,7 @@ def plot_streams(
         cmap: Name of the matplotlib colormap to use.
         levels: Number of contour levels to used.
         colorbar: Whether to add a colorbar to each subplot.
+        filled: If True, plots filled contours.
 
     Returns:
         matplotlib figure and axes
@@ -341,6 +346,7 @@ def plot_streams(
             cmap=cmap,
             levels=levels,
             colorbar=colorbar,
+            filled=filled,
         )
         used_axes.append(ax)
         if cbar is not None:
@@ -572,12 +578,28 @@ def plot_currents(
         method=grid_method,
         layers=layers,
     )
+    # Create masks to set the current density to zero in holes
+    # and outside of films.
+    # This is really only required for cubic interpolation, but doesn't hurt
+    # to do in all cases.
+    points = np.stack([xgrid.ravel(), ygrid.ravel()], axis=1)
+    film_masks = device.contains_points_by_layer(
+        points,
+        polygon_type="film",
+    )
+    hole_masks = device.contains_points_by_layer(
+        points,
+        polygon_type="hole",
+    )
     jcs = {}
     Js = {}
     for layer, jc in current_densities.items():
+        mask = ~(film_masks[layer] & ~hole_masks[layer]).reshape(xgrid.shape)
         jc = jx, jy = (jc * old_units).to(units).magnitude
+        jc[:, mask] *= 0
         jcs[layer] = jc
         Js[layer] = np.sqrt(jx**2 + jy**2)
+
     clabel = "$|\\,\\vec{J}\\,|$" + f" [${units:~L}$]"
     clim_dict = setup_color_limits(
         Js,
@@ -968,6 +990,7 @@ def plot_polygon_flux(
 
 def _patch_docstring(func):
     other_func = getattr(Solution, func.__name__)
+    other_func.__signature__ = inspect.signature(func)
     other_func.__doc__ = (
         other_func.__doc__
         + "\n\n"
